///****************************************************************************
//// Name:        DFTTSSampleCPP with DFTTS Desktop SDK v 3.5.0
//// Purpose:
//// Author:      Digital Future
//// Modified by:
//// Created:     06/19/2008, updated 07/25/2008
//// RCS-ID:
//// Copyright:   (C) Digital Future 2007-2008. All rights reserved.
//// License:     see license document
//***************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include "dftts.h"

#define DICT_NAME "cepstral_dictionary.txt"
#define VOICE_NAME "David"
#define ENGINE_TYPE CEPSTRAL
#define LANG_TYPE LANG_SUBLANG_NEUTRAL
#define AUDIO_FILE_NAME "test.wav"
#define AUDIO_FORMAT 0 //Only used with NEOSPEECHVOICETEXT and MSSAPI (Windows)
#define AUDIO_FORMAT_STR "pcm16"
#define AUDIO_SAMPLING_RATE 16000
#define AUDIO_CHANNELS 2


pthread_mutex_t condition_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t  condition_cond  = PTHREAD_COND_INITIALIZER;

void* SpeakText( void *text )
{

	/*Cepstral David*/	
		
	//This is how to load a dictionary for the Cepstral provider (the dictionary is loaded 
	//by voice, the dictionary number, dict name, dict content do NOT matter.)

	LoadDFTTSUserDictReturnValue dictreturn = LoadDFTTSUserDict( -1, NULL,
						DICT_NAME,
						ENGINE_TYPE, LANG_TYPE, VOICE_NAME, NULL );	
													
	DFTTSSpeakReturnValue result =  DFTTSSpeak( 0,ENGINE_TYPE,VOICE_NAME,-1,
					LANG_TYPE,(char*)text,-1,-1,-1,-1,
					-1,DFTTS_TEXT_TYPE_XML,-1 );


	if( dictreturn != LOAD_USER_DICT_SUCCESS )
	{
		char szMsg[256];
			
		sprintf( szMsg, "Dictionary load error: %d\n", (short)dictreturn );
			
		printf(szMsg);
			
	}
		
	if( result != DFTTS_SPEAK_SUCCESS )
	{
		char szMsg[256];
			
		sprintf( szMsg, "SDK Speak error: %d\n", (short)result );
			
		printf(szMsg);
			
	}

      	pthread_mutex_lock( &condition_mutex );
      	
        pthread_cond_wait( &condition_cond, &condition_mutex );
      
		pthread_mutex_unlock( &condition_mutex );	

     	return NULL;
	
}

void* ExportText( void *text )
{
	
	
	/*Cepstral David*/	
	DFTTSExportReturnValue expresult = DFTTSExportToFileEx
		  ( ENGINE_TYPE,VOICE_NAME,-1,
		 LANG_TYPE,(char*)text,
		 -1,-1,-1,-1,-1,DFTTS_TEXT_TYPE_XML,AUDIO_FILE_NAME, AUDIO_FORMAT, 
			AUDIO_FORMAT_STR, AUDIO_SAMPLING_RATE, AUDIO_CHANNELS );


	if( expresult != DFTTS_EXPORT_SUCCESS )
	{
		char szMsg[256];
				
		sprintf( szMsg, "SDK Export error: %d\n", (short)expresult );
				
		printf(szMsg);
				
	}
	
	return NULL;
}

void callBackOnWord( int start, int end )
{
		
	if( end==-1 )		
		pthread_cond_broadcast( &condition_cond );

	char szMsg[256];
	
	sprintf( szMsg, "Start character: %d. End character: %d.\n", start, end );

	printf( szMsg );

}

void callBackOnExport( int start, int end )
{

	char szMsg[256];
	
	sprintf( szMsg, "Start exp. character: %d. End exp. character: %d.\n", start, end );

	printf( szMsg );

}

int main(int argc, char *argv[])
{

	//Initilize voices and set the callback
	
	//Set the OnWord callback function
	
	SetDFTTSOnWordCallBack( callBackOnWord );
				
	//Set the OnExport callback function
				
	SetDFTTSOnExportCallBack( callBackOnExport );
	

  	InitDFTTSEngineReturnValueEx* initresult = InitDFTTSEngineEx( 0,0,0 );
	
				
	/****************************************************

	initresult is an pointer to InitDFTTSEngineReturnValueEx with 2 array members.

	//NEOSPEECHVOICETEXT ENGLISH (NOT SUPPORTED BY MAC OS X and Linux)
	initresult.vet[0] = NEOSPEECHVOICETEXT; //KATE
	initresult.vet[1] = NEOSPEECHVOICETEXT; //PAUL

	//NEOSPEECHVOICETEXT JAPANESE
	initresult.vet[2] = NEOSPEECHVOICETEXT; //MIYU
	initresult.vet[3] = NEOSPEECHVOICETEXT; //SHOW
	initresult.vet[4] = NEOSPEECHVOICETEXT; //MISAKI

	//NEOSPEECHVOICETEXT CHINESE
	initresult.vet[5] = NEOSPEECHVOICETEXT; //LILY
	initresult.vet[6] = NEOSPEECHVOICETEXT; //WANG

	//NEOSPEECHVOICETEXT KOREAN
	initresult.vet[7] = NEOSPEECHVOICETEXT; //JUNWOO
	initresult.vet[8] = NEOSPEECHVOICETEXT; //SUJIN
	initresult.vet[9] = NEOSPEECHVOICETEXT; //YUMI
	initresult.vet[10] = NEOSPEECHVOICETEXT; //GYURI
	initresult.vet[11] = NEOSPEECHVOICETEXT; //DAYOUNG

	//CEPSTRAL
	initresult.vet[12] = CEPSTRAL;

	//AT&T NV (NOT SUPPORTED BY MAC OS X and Linux)
	initresult.vet[13] = ATTNV; 
	
	//MAC OS X SPMAN (NOT SUPPORTED BY Windows and Linux)
	initresult.vet[14] = MACOSXSPMAN; 
	
	//MICROSOFT SAPI (NOT SUPPORTED BY MAC OS X and Linux)
	initresult.vet[15] = MSSAPI; 				

	ert memebers are of type InitDFTTSEngineReturnValue
	
	(NOT SUPPORTED BY Linux and Mac OS X)
	initresult.ert[0] - error code for NeoSpeech Kate
	initresult.ert[1] - error code for NeoSpeech Paul
	initresult.ert[2] - error code for NeoSpeech Miyu
	initresult.ert[3] - error code for NeoSpeech Show
	initresult.ert[4] - error code for NeoSpeech Misaki
	initresult.ert[5] - error code for NeoSpeech Lily
	initresult.ert[6] - error code for NeoSpeech Wang
	initresult.ert[7] - error code for NeoSpeech Junwoo
	initresult.ert[8] - error code for NeoSpeech Sujin
	initresult.ert[9] - error code for NeoSpeech Yumi
	initresult.ert[10] - error code for NeoSpeech Gyuri
	initresult.ert[11] - error code for NeoSpeech Dayoung
	
	initresult.ert[12] - error code for the whole Cepstral engine
	
	(NOT SUPPORTED BY Linux and Mac OS X)
	initresult.ert[13] - error code for the whole AT&T NV engine
	
	(NOT SUPPORTED BY Linux and Windows)	
	initresult.ert[14] - error code for the whole MAC OS X SPMAN
	
	(NOT SUPPORTED BY Linux and Mac OS X)				  				  
	initresult.ert[15] - error code for the whole MS SAPI

	*****************************************************/
	
	if( initresult->ert[12] != INIT_DFTTS_ENGINE_SUCCESS )
	{
		
		char szErrNo[2];
		
		sprintf( szErrNo, "%d", (short)initresult->ert[12] );
		
		printf("CEPSTRAL engine initialization error: ");
		
		printf( szErrNo );
		
		printf( ".\nThe voice or engine may not be installed.\n" );
		
		
	}
	
	//This is how to list all installed supported voices


	char szVoiceName[1024];

	int iVoiceNameLen = 0;

	DFTTSVoiceEngineType vet = NOENGINE;

	unsigned short lang = LANG_SUBLANG_NEUTRAL;

	int iVoiceID = -1;

	int z=0;

	for( ; ; z++ )
	{
		DFTTSVoiceInfoReturnValue result = GetDFTTSVoice(
				z, &vet, szVoiceName, &iVoiceNameLen,
				&iVoiceID, &lang );


		if( result == GET_DFTTS_VI_SUCCESS )
		{

		
			printf( szVoiceName );
			printf( "\n" );


		}
		else
		{
			break;
		}


	}

	if( z == 0 )
	{
		printf( "No voices can be found on your system! Please download and install at least 1 voice!\n" );
		exit(1);
	}

	
	char* szText = "Testing the Digital Future Text-to-Speech SDK.";
	pthread_t thread1, thread2;
	
	int iret1 = pthread_create( &thread1, NULL, SpeakText, (void*) szText);
     	
	pthread_join( thread1, NULL);

	int iret2 = pthread_create( &thread2, NULL, ExportText, (void*) szText);

     	pthread_join( thread2, NULL); 

     	printf("Speak Thread returned: %d\n",iret1);
     	printf("Export Thread returned: %d\n",iret2);

	//IMPORTANT!
	UninitDFTTSEngine(); 

	exit(0);

	



}
